
local object = {}
object.obj = {}
object.max = 50
object.id = 0
world = love.physics.newWorld(0, 9.16*81, true)

function object.newRectangle(type, x, y, w, h, userdata)
	object.id = object.id + 1
	local obj = {}
	obj.x, obj.y = x, y
	obj.w, obj.h = w, h
	obj.shapee = "rectangle"
	obj.name = userdata
	obj.type = type
	obj.dead = false
	obj.rgb = {r, g, b}
	obj.body = love.physics.newBody(world, x, y, type)
	obj.shape = love.physics.newRectangleShape(w, h)
	obj.fixture = love.physics.newFixture(obj.body, obj.shape)
	obj.fixture:setUserData(userdata)
	function obj:createJoint(x, y)
		self.joint = love.physics.newMouseJoint(self.body, x, y) 
	end
	function obj:removeJoint()
		if self.joint then
			self.joint:destroy()
			self.joint = false
		end
	end
	function obj:remove()
		if not self.dead then
			self.body:destroy()
			self.dead = true
			object.id = object.id - 1
		end
	end
	table.insert(object.obj, obj)
end

function object.newCircle(type, x, y, rad, userdata)
	object.id = object.id + 1
	local obj = {}
	obj.x, obj.y = x, y
	obj.type = type
	obj.dead = false
	obj.shapee = "circle"
	obj.rad = rad
	obj.name = userdata
	obj.rgb = {r, g, b}
	obj.body = love.physics.newBody(world, x, y, type)
	obj.shape = love.physics.newCircleShape(rad)
	obj.fixture = love.physics.newFixture(obj.body, obj.shape)
	obj.fixture:setUserData(userdata)
	function obj:createJoint()
		self.joint = love.physics.newMouseJoint(self.body, self.body:getX(), self.body:getY()) 
	end
	function obj:removeJoint()
		if self.joint then
			self.joint:destroy()
			self.joint = false
		end
	end
		function obj:remove()
		if not self.dead then
			self.body:destroy()
			self.dead = true
			object.id = object.id - 1
		end
	end
	table.insert(object.obj, obj)
end

function object.clear()
	for k, v in ipairs(object.obj) do
		v:remove()
		object.id = 0
	end
end

function object.clearOffscreen()
	for k, v in ipairs(object.obj) do
		if not v.dead then
			if v.shapee == "rectangle" then
				if not col(v.body:getX(), v.body:getY(), v.w, v.h, camera._x, camera._y, config.width, config.height) then
					v:remove()
				end
			elseif v.shapee == "circle" then
				if not col(v.body:getX(), v.body:getY(), v.rad, v.rad, camera._x, camera._y, config.width, config.height) then
					v:remove()
				end
			end
		end
	end
end

function object.draw()
	for k, v in ipairs(object.obj) do
		love.graphics.setColor(v.rgb)
		if not v.dead then
			if v.shapee == "rectangle" then
				love.graphics.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))
			elseif v.shapee == "circle" then
				love.graphics.circle("fill", v.body:getX(), v.body:getY(), v.shape:getRadius())
			end
		end
	end
end

function object.update(dt)
	world:update(dt)
	for k, v in ipairs(object.obj) do
		if v.joint then
			v.joint:setTarget(camera:mousePosition())
		end
	end
end

function object.mousepressed(x, y, b)
	local x, y = camera:mousePosition()
	if mode == "Drag" and can == true then
		if b == "l" then
			for k, v in ipairs(object.obj) do
				if not v.dead then
					local inside = v.fixture:testPoint(x, y)
					if inside then
						if not v.joint then
							v:createJoint(x, y)
						end
					end
				end
			end
		end
	end
end

function object.mousereleased(x, y, b)
	if mode == "Drag" and can == true then
		if b == "l" then
			for k, v in ipairs(object.obj) do
				if not v.dead then
					if v.joint then
						v:removeJoint()
					end
				end
			end
		end
	end
end

return object